<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli	<marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('system/CronManager.php');
 
/**
 * GForge daemon. 
 * The daemon is started by the "gforge" script, and is accessed by the individual static methods.
 */
class GFDaemon {
	/** PID file */
	const PID_FILE = '/tmp/gforge.pid';
	const WAKEUP_TIME = 60;		// Spawn daemon every minute
	
	/** Do not instantiate directly */
	private function __construct() {
	}

	public static function start() {
		if (!function_exists("posix_setgid")) {
			throw new Exception("POSIX functions not present in PHP");
		}
		if (!function_exists("pcntl_fork")) {
			throw new Exception("pcntl_fork function not present in PHP. Please recompile with --enable-pcntl.");
		}
		if (self::isRunning()) {
			throw new Exception("GForge daemon is already running");
		}

		$userData = posix_getpwuid(posix_getuid());
		if ($userData["name"] != "root") {
			throw new Exception("Must be root for running the daemon");
		}
		
		if (!self::fork()) {
			throw new Exception("Couldn't fork process");
		}
		
	}
	
	public static function stop() {
		if (!self::isRunning()) {
			throw new Exception("GForge daemon is not running");
		}
		
		$pid = self::getDaemonPid();
		if (!posix_kill($pid, SIGTERM)) {
			throw new Exception("Could not kill PID $pid");
		}
	}
	
	public static function isRunning() {
		if (file_exists(self::PID_FILE) && ($pid = @file_get_contents(self::PID_FILE))) {
			// this will tell if process is running without actually killing it
			return posix_kill($pid, 0);
		} else {
			return false;
		}
	}
	
	/**
	 * Returns the PID of the daemon
	 */
	public static function getDaemonPid() {
		if (!file_exists(self::PID_FILE)) {
			return 0;
		}
		
		$pid = file_get_contents(self::PID_FILE);
		return $pid;
	}
	
	/**
	 * Begins the daemon process
	 */
	private static function fork() {
		$pid = pcntl_fork();
		if ($pid == -1) {
			return false;
		} else if ($pid != 0) {
			// We're on parent process
			return true;
		}
		
		// Here we are on the forked process
		self::handleDaemon();
	}
	
	public static function handleSignal($signal) {
		// Clean up
		unlink(self::PID_FILE);
		exit(0);
	}
	
	/**
	 * Handler for the daemon process
	 */
	private static function handleDaemon() {
		// Install the signal handler
		declare(ticks = 1);		// Somehow this is needed... else it won't work
		$signals = array(SIGTERM, SIGHUP, SIGQUIT);
		foreach ($signals as $signal) {
			if (!pcntl_signal($signal, array("self", "handleSignal"))) {
				echo "Could not install signal handler. Aborting.\n";
				exit(1);
			}
		}

		// get the daemon's PID and write it to the PID file
		$pid = posix_getpid();
		$fh = fopen(self::PID_FILE, "w");
		if (!$fh) {
			echo "Could not open ".self::PID_FILE." for writing. Aborting.\n";
			exit(1);
		}

		fwrite($fh, $pid);
		fclose($fh);
		
		self::daemonLoop();
	}
	
	/**
	 * Main daemon loop
	 */
	private static function daemonLoop() {
		$cronManager = new CronManager();
		while (true) {
			sleep(self::WAKEUP_TIME);		// Spawn every minute
			// Process!
			$cronManager->processCrons();
		}
	}
}
?>