<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   
 * @author Daniel A. Perez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This file is the main wiki engine, which provides all wiki functionality

require_once('external/fckeditor/fckeditor.php');
require_once('plugins/userwiki/lib/WikiPagePeer.php');
require_once('plugins/userwiki/lib/WikiPageLinkPeer.php');
require_once('plugins/userwiki/lib/UserwikiPlugin.php');
require_once('lib/core/FilesystemPeer.php');
require_once('lib/StringUtils.php');

class UserWiki {
	
	private $section;
	private $ref_id;
	private $pagename; // that came as a parameter or from a loaded page object
	private $is_admin = false;
	private $page; // page object
	private $page_version; // page version object
	private $theme; // theme object
	private $user; // user object
	
	// Note: this file MUST be saved in UTF8 encoding in order for this to work!
	const WikiWordChars = "A-ZÁÉÍÓÚÄËÏÖÜÀÈÌÒÙÂÊÎÔÛÇÑa-záéíóúäëïöüàèìòùâêîôûçñ";
	const WikiWordRegex = "\b(([A-ZÁÉÍÓÚÄËÏÖÜÀÈÌÒÙÂÊÎÔÛÇÑ][a-záéíóúäëïöüàèìòùâêîôûçñ]+){2,})"; // WikiWordsAreLikeThis
	
	function __construct($section,$ref_id,$theme,$user) {
		$this->section = $section;
		$this->ref_id = $ref_id;
		$this->theme = $theme;
		$this->user = $user;
		$pagename = null;
		$page = null;
	}
	
	
	/**
	 * Gets from the DB the information of the page indicated by the page id
	 *
	 * @param int	page id
	 * @return html ready page or null if not found
	 */
	public function retrievePageById($id) {
		$this->page = UserWikiPagePeer::retrieveByPK($id);
		if ( (!is_object($this->page)) || (!($this->ref_id == $this->page->getRefId())&&($this->section == $this->page->getSection())  )) { // page not found OR page isn't from designed ref_id + section
			$this->pagename = null;
			$this->page = null;
			return null;
		} else {
			$this->pagename = $this->page->getPagename();
		}
		return $this->page;
	}
	
	/**
	 * Gets from the DB the information of the page using its name and references
	 *
	 * @param string 	pagename
	 * @return html ready page or null if not found
	 */
	public function retrievePageByName($pagename) {
		$c = new Criteria();
		$c->add(UserWikiPagePeer::PAGENAME,$pagename);
		$c->add(UserWikiPagePeer::SECTION,$this->section);
		$c->add(UserWikiPagePeer::REF_ID,$this->ref_id);
		$this->page = UserWikiPagePeer::doSelectOne($c);
		if (!is_object($this->page)) { // page not found
			$this->pagename = null;
			$this->page = null;
			return null;
		} else {
			$this->pagename = $this->page->getPagename();
		}
		return $this->page;
	}	
	
	/**
	 * Contains the logic to search for the frontpage of the wiki, given the ref_id and section
	 *
	 * @param string	message (optional, to display in header)
	 * @return html ready page
	 */
	public function displayFrontPage($message='') {
		$this->retrievePageByName('FrontPage'); // the frontpage is always a default. 
		if (!is_object($this->page)) {
			return $this->generateDefaults();
		} else {
			if ($this->page->userCanRead()) {
				$contents = $this->page->getLatestContents();
				// increase the hit count
				$this->page->increaseHitCount($this->page->getLatestVersion()->getPageVersion());
			} else {
				$return = $this->generateWikiHeader('Permission Denied');
				$return .= $this->generateWikiFooter();
				return $return;				
			}
		}
		$this->page_version = $this->page->getLatestVersion();
		return $this->generatePage($contents,$message);
		
	}

	/**
	 * searchs for the page and returns its content
	 *
	 * @param string 	pagename
	 * @param int	version	(page_version) defaults to latest version
	 * @param string	message	(optional, displayed in the header)
	 * @return html ready page
	 */
	public function displayPage($pagename,$page_version=null,$message='',$parent=null) {
		if ($this->retrievePageByName($pagename)) {
			if ($this->page->userCanRead()) {
				// get contents for latest version
				if ($page_version) {
					$contents = $this->page->getContents($page_version);
					$this->page_version = $this->page->getVersion($page_version);
					if(is_null($this->page_version)) {
						throw new InvalidDataException(array('Non valid page version'));
					}
					// increase the hit count
					$this->page->increaseHitCount($this->page->getVersion($page_version)->getPageVersion());
				} else {
					$contents = $this->page->getLatestContents();
					$this->page_version = $this->page->getLatestVersion();
					if(is_null($this->page_version)) {
						throw new InvalidDataException(array('Non valid page version'));
					}
					// increase the hit count
					$this->page->increaseHitCount($this->page->getLatestVersion()->getPageVersion());
				}
				$result = $this->generatePage($contents,$message);
			} else {
				$return = $this->generateWikiHeader('Permission Denied');
				$return .= $this->generateWikiFooter();
				return $return;
			}
		} else {
			$dummy = new UserWikiPage();
			$dummy->setRefId($this->ref_id);
			$dummy->setSection('user');
			if ($dummy->userCanCreate()) {
				// page doesn't exist, display create form
				if (strlen($pagename)>2) {
					$contents = Language::getSessText('UserWiki.NewPageContents',array($pagename));
					$this->pagename = $pagename;
					$result = $this->generateEditForm($contents,true,$parent);					
				} else {
					$return = $this->generateWikiHeader('Pagename too short, must be at least 3 characters');
					$return .= $this->generateWikiFooter();
					return $return;
				}
			} else {
				$return = $this->generateWikiHeader('Permission Denied');
				$return .= $this->generateWikiFooter();
				return $return;
			}
		}
		return $result;
	}

	/**
	 * searchs for the page and returns its content
	 *
	 * @param string 	pagename
	 * @param int	version	(page_version) defaults to latest version
	 * @param string	message	(optional, displayed in the header)
	 * @return html ready page
	 */
	public function displayPageById($page_id,$page_version=null,$message='') {
		if ($this->retrievePageById($page_id)) {
			if ($this->page->userCanRead()) {
				if ($page_version) {
					$contents = $this->page->getContents($page_version);
					$this->page_version = $this->page->getVersion($page_version);
					// increase the hit count
					$this->page->increaseHitCount($this->page->getVersion($page_version)->getPageVersion());				
				} else {
					$contents = $this->page->getLatestContents();
					$this->page_version = $this->page->getLatestVersion();
					// increase the hit count
					$this->page->increaseHitCount($this->page->getLatestVersion()->getPageVersion());				
				}
				return $this->generatePage($contents,$message);
			} else {
				$return = $this->generateWikiHeader('Permission Denied');
				$return .= $this->generateWikiFooter();
				return $return;				
			}
		} else {
			// trying to access by an id which doesn't correspond to the section + ref_id, or else it just doesn't exist
			return ('<font color=red>'.Language::getSessText('UserWiki.NoAccessThisId').'</font>');
		}
	}
	
	/**
	 * Generates the html form for an edited page
	 *
	 * @param string $contents
	 * @param boolean $isnew -> whether the page is new or we're editing an existing one
	 */
	private function generateEditForm($contents,$isnew,$parent=null) {
		
		if (!$isnew) { // if it's a new page the perm check is done before ever entering here
			if (!$this->page->userCanUpdate()) {
				$return = $this->generateWikiHeader('Permission Denied');
				$return .= $this->generateWikiFooter();
				return $return;
			}
		}
		// setting the fckeditor object
		$oFCKeditor = new FCKeditor('content') ;
		// IMPORTANT TODO -> fckeditor needs a reference to the url. to accomplish this, we must create a link from the root www dir (where gf resides) to
		// the directory where fckeditor is (for example lib/external/fckeditor and name it as that dir (fckeditor)
		$oFCKeditor->BasePath = GFUrl::getUrl()->buildURL('/fckeditor/');/* 'http://gforge5/fckeditor/';*/
		$oFCKeditor->Value = $contents; // this is the initial text that will be displayed (if any)
		$oFCKeditor->Width = "800";
		$oFCKeditor->Height = "500";
		$oFCKeditor->ToolbarSet = "GForge";
		//$oFCKeditor->setConnectorURLs($this->project->getProjectId());				
		$value = $isnew?0:$this->page->getLatestVersion()->getPrimaryKey();
		$is_new = $isnew?'true':'false';
		require_once('plugins/'. UserwikiPlugin::SECTION .'/wwwlib/forms/EditForm-tpl.php');
		return $result;
	}
	
	/**
	 * Tries to fetch the page from the DB to edit it, or otherwise creates a new one
	 *
	 * @param string	$editpage -> page name
	 */
	public function editPage($editpage) {
		if ( (!is_object($this->page)) || ( ($pagename) && ($pagename != $editpage) )) {
			if ($this->retrievePageByName($editpage)) {
				if ($this->page->getType()=='text/plain') {
					$result = $this->generateEditForm(nl2br($this->page->getLatestContents()),false); // edit the page
				} else {
					$result = $this->generateEditForm($this->page->getLatestContents(),false); // edit the page
				}
			} else { //create new one
				$contents = Language::getSessText('UserWiki.NewPageContents',array($editpage));
				$this->pagename = $editpage;
				$result = $this->generateEditForm($contents,true);			
			}
		} else {
			$result = $this->generateEditForm($this->page->getLatestContents,false);
		}
		return $result;
	}
	
	/**
	 * Create new page
	 *
	 * @param string $name
	 * @param string	page contents
	 * @return int	the newly created page id
	 */
	public function createNewPage($name,$content) {
		$user_id = Session::getSession()->getUser()->getUserId();
		try {
			$con = Propel::getConnection(WikiPagePeer::DATABASE_NAME);
			$con->begin();
			
			// create page
			$this->page = new UserWikiPage();
			$this->page->setSection($this->section);
			$this->page->setRefId($this->ref_id);
			$this->page->setPagename($name);
			$this->page->setFlags(WikiPage::FLAGS_UNLOCKED);
			$this->page->setLatestVersion(1); // first version
			$this->page->save();
			//create version
			$page_version = new WikiVersion();
			$page_version->setWikiPageId($this->page->getPrimaryKey());
			$page_version->setPageVersion(1);
			$page_version->setPostedBy($user_id);
			$page_version->setRefs('a:0:{}'); // TODO -> how does this work?
			$time = date('d-m-Y-G-i-s'); // get current time
			$time = explode('-',$time);
			$lastmod = $time[2] . "-" . $time[1] . "-" . $time[0] . " " . $time[3] . ":" . $time[4] . ":" . $time[5];		
			$page_version->setLastModifiedDate($lastmod);
			$page_version->setScore(0);
			//$page_version->setHits(0);
			$page_version->setCreateDate($lastmod);
			$page_version->save();
			//save contents to filesystem
			$pagename = $this->page->getPageName();
			$fs = new Filesystem();
			$fs->setFileName($pagename);
			$fs->setPostedBy($user_id);
			$fs->setDeleted(false);
			//$fs->setFileNameSafe($pagename);
			$fs->setFileType('text');
			$fs->setRefId($page_version->getPrimaryKey());
			$fs->setSection(UserwikiPlugin::SECTION);
			$fs->save();
			$fs->setFileDataFromString($content,true);
			$this->pagename = $pagename;
			$id = $this->page->getPrimaryKey();
			// CREATE LINKS
			$this->updateLinks($this->page,$content);
			// TODO -> for the pages that link to this new page, update the links
			
			$con->commit();
			
			return $id;
		} catch (Exception  $e) {
			echo $e->getMessage();
			return -1;
		}
	}

	/**
	 * Update an already existing page
	 *
	 * @param int $editversion -> the version id
	 * @param string $content -> the new contents
	 * @param boolean	whether to archive old version or not
	 */
	public function updatePage($editversion,$content,$archive) {
		
		$page_version = UserWikiVersionPeer::retrieveByPK($editversion);
		$page = $this->retrievePageById($page_version->getWikiPageId());
		
		$con = Propel::getConnection(WikiPagePeer::DATABASE_NAME);
		$con->begin();

		if ($archive) {
			// leave the old version and create a new one, and a new file	
			$new_version = new WikiVersion();
			$new_version->setWikiPage($page);
			$new_version->setPageVersion($page_version->getPageVersion() + 1);
			$user_id = Session::getSession()->getUser()->getUserId();
			$new_version->setPostedBy($user_id);
			$new_version->setRefs('a:0:{}'); // TODO -> how does this work?
			$time = date('d-m-Y-G-i-s'); // get current time
			$time = explode('-',$time);
			$lastmod = $time[2] . "-" . $time[1] . "-" . $time[0] . " " . $time[3] . ":" . $time[4] . ":" . $time[5];
			$new_version->setLastModifiedDate($lastmod);
			$new_version->setScore(0);
			//$new_version->setHits(0);
			$new_version->setCreateDate($lastmod);
			$page->setLatestVersion($page_version->getPageVersion());
			$page->save();
			$new_version->save();
			$ref_id = $new_version->getPrimaryKey();
			$pagename = $page->getPageName();
			$new_file = new Filesystem();
			$new_file->setFileName($pagename);
			$new_file->setPostedBy($user_id);
			$new_file->setDeleted(false);
			//$new_file->setFileNameSafe($pagename);
			$new_file->setFileType('text/html');
			$new_file->setRefId($ref_id);
			$new_file->setSection(UserwikiPlugin::SECTION);
			$new_file->save();
			$new_file->setFileDataFromString($content,true);
		} else {
			// just update the current version and file with the new values
			$page_version->setPageVersion($page_version->getPageVersion() + 1);
			$page_version->setPostedBy(Session::getSession()->getUser()->getUserId());
			$page_version->setRefs('a:0:{}'); // TODO -> how does this work?
			$time = date('d-m-Y-G-i-s'); // get current time
			$time = explode('-',$time);
			$lastmod = $time[2] . "-" . $time[1] . "-" . $time[0] . " " . $time[3] . ":" . $time[4] . ":" . $time[5];
			$page_version->setLastModifiedDate($lastmod);
			//$page_version->setHits($page_version->getHits() + 1);
			$page_version->save();
			$ref_id = $page_version->getPrimaryKey();
			$c = new Criteria();
			$c->add(FilesystemPeer::SECTION,UserwikiPlugin::SECTION);
			$c->add(FilesystemPeer::REF_ID,$ref_id);
			$fso = FilesystemPeer::doSelectOne($c);
			$fso->setFileDataFromString($content,true);
			$page->setLatestVersion($page_version->getPageVersion());
			$page->save();
		}
		// update the links from this file to other files
		$this->updateLinks($page,$content);
		$page_id = $page->getPrimaryKey();
		$con->commit();
		return $page_id;
	}

	/**
	 * Updates the links for a page
	 *
	 * @param WikiPage $page
	 * @param string $contents
	 */
	
	public function updateLinks($page,$content) {
		$page_id = $page->getPrimaryKey();
		$c = new Criteria();
		$c->add(UserWikiPageLinkPeer::FROM_WIKI_PAGE_ID,$page_id);
		$old_links = UserWikiPageLinkPeer::doSelect($c);
		foreach ($old_links as $old_link) {
			$old_link->delete(); // remove the old links
		}
		$new_links_names = $this->extractLinks($content);
		foreach ($new_links_names as $new_link_name) {
			$curr_page = $this->retrievePageByName($new_link_name);
			if ($curr_page instanceof UserWikiPage) { // perhaps we have a link to a yet unexisting page
				$link = new UserWikiPageLink();
				$link->setFromWikiPageId($page_id);
				$link->setToWikiPageId($curr_page->getPrimaryKey());
				$link->save();
				unset($link);
			}
		}
		// update the score for the page's latest version
		$con = Propel::getConnection(UserWikiPagePeer::DATABASE_NAME);
		$sql = 'select distinct w1.from_wiki_page_id, count(*) as qty from wiki_page_link as w1, wiki_page_link as w2 where 
				w1.from_wiki_page_id=w2.to_wiki_page_id and w1.from_wiki_page_id='.$page_id.' group by w1.to_wiki_page_id,w1.from_wiki_page_id';
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		if ($rs->next()) {
			$latest_ver = $page->getLatestVersion();
			if (is_object($latest_ver)) {
				$latest_ver->setScore($rs->getInt('qty'));
				$latest_ver->save();
			}
		}		
	}
	
	/**
	 * searches the string and returns an array with the wiki links found
	 *
	 * @param unknown_type $contents
	 */
	
	private function extractLinks($contents) {
		$wikilinks = array();
		$content_array = explode('<br />',$contents); 
		$linecount = count($content_array);

		$disableBracketLinks = GFConf::get('userwiki.disableBracketLinks');
		$disableWikiWords = GFConf::get('userwiki.disableWikiWords');

		for($l = 0; $l < $linecount; $l++) {
			// remove escaped '['
			$line = str_replace('[[', ' ', $content_array[$l]);

			// bracket links
			if (!$disableBracketLinks) {
				$numBracketLinks = preg_match_all("/[\\\\]?\[\s*([^\]|]+\|)?\s*(.+?)\s*\]/", $line, $brktlinks);
				$brktlinks = $this->removeEscaped($brktlinks, false);
				for ($i = 0; $i < count($brktlinks); $i++) {
					if (isset($brktlinks[2][$i])) {
						$wikilinks[] = $brktlinks[2][$i];
					}
					if (isset($brktlinks[0][$i])) {
						$brktlink = preg_quote($brktlinks[0][$i]);
						$line = preg_replace("|$brktlink|", '', $line);
					}
				}
			}
			
			//WikiWord links
			if (!$disableWikiWords) {
				if (preg_match_all("/(\\\)?".Wiki::WikiWordRegex."/", $line, $link)) {
					$link = $this->removeEscaped($link, false);
					for ($i = 0; isset($link[0][$i]); $i++) {
						if($link[0][$i][0] <> '!') {
							$wikilinks[] = $link[0][$i];
						}
					}
				}
			}
		}
		$wikilinks = array_unique($wikilinks); // remove duplicates
		return $wikilinks;			
	}

	
	/**
	 * Transforms the contents of the page and parses wikilinks, etc
	 *
	 * @param string $contents
	 */
	private function  Transform($contents) {
		// Loop over all lines of the page and transform
		$content_array = explode('<br />',$contents); 
		$linecount = count($content_array);
		$html = '';
		for ($index = 0; $index < $linecount; $index++) {
			$tmp = $this->HandleWikiWords($content_array[$index]);

			// some GF4 wiki constants defined in some pages, for backwards compatibility
			if (strstr($tmp, '%%Search%%')) {
				$tmp = str_replace('%%Search%%', $this->getQuickSearchForm(), $tmp);
			}
			if (strstr($tmp, '%%Fullsearch%%')) {
				$tmp = str_replace('%%Fullsearch%%', $this->getFullSearchForm(), $tmp);						
			}
			if (strstr($tmp, '%%Mostpopular%%')) {
				$tmp = str_replace('%%Mostpopular%%', $this->getMostPopular(), $tmp);						
			}
			$html .= $tmp . '<br />';
		}
		return $html;
	}

	/**
	 * handles wikiwords
	 *
	 * @param string $text
	 * @return string
	 */

	private function HandleWikiWords($text) {
		$LinksRegex = '/\<a([^>]*)\>(.*?)\<\/a\>/i';
		$HTMLTagsRegex = "/<[^<>]*>/i";
		$BracketsRegex = '/(\\\?\[.+?\])/';
		$EscBracketsRegex = '/([\\\]\[.+?\])/';
		$URLsRegex = '/([^"\'])(http|ftp|https)([\:\/\/])([^\\s<]+)([^"\'])/';

		$LinkMarker = '$$##$$';
		$HTMLMarker = '$$!!$$';
		$BracketMarker = '$$@@$$';
		$WikiMarker = '$$%%$$';
		$URLMarker = '$$||$$';

		$disableBracketLinks = GFConf::get('wiki.disableBracketLinks');
		$disableWikiWords = GFConf::get('wiki.disableWikiWords');
		
		$result = $text;

		// Convert non-HTML URLs to HTML Links (this must be done first)
		preg_match_all($URLsRegex, $result, $URLs);
		$result = preg_replace($URLsRegex, $URLMarker, $result);

		preg_match_all($LinksRegex, $text, $Links);
		$result = preg_replace($LinksRegex, $LinkMarker, $result);
		
		preg_match_all($HTMLTagsRegex, $result, $HTMLTags);
		$result = preg_replace($HTMLTagsRegex, $HTMLMarker, $result);

		if (!$disableBracketLinks) {
			preg_match_all($BracketsRegex, $result, $BracketWords);
			$BracketWords[0] = $this->removeEscaped($BracketWords[0], true);
			//$result = str_replace($BracketWords[0], $BracketMarker, $result);
			for ($i = 0; $i < count($BracketWords[0]); $i++) {
				// This prevents a bug where having an escaped bracketed sentence
				// and the same non-escaped bracketed sentence on the same line
				// screws up the output
				$result = preg_replace('/([^\\\\])'.preg_quote($BracketWords[0][$i], '/').'/', '$1'.$BracketMarker, $result);
			}
		}
		
		if (!$disableWikiWords) {
			preg_match_all("/(\\\)?".self::WikiWordRegex.'/', $result, $WikiWords);
			$WikiWords[0] = $this->removeEscaped($WikiWords[0], true);
			for ($i = 0; $i < count($WikiWords[0]); $i++) {
				// This fixes the same bug mentioned above with the WikiWords
				//$result = preg_replace('/([^\\\\])'.preg_quote($WikiWords[0][$i]).'/', '$1'.$WikiMarker, $result);
				$result = preg_replace('/([^\\\\'.self::WikiWordChars.']?)'.preg_quote($WikiWords[0][$i], '/').'([^'.self::WikiWordChars.']?)/', '$1'.$WikiMarker.'$2', $result);
			}
			
		}

		if (!$disableWikiWords) {
			foreach($WikiWords[0] as $WikiWord) {
				$result = StringUtils::strReplaceOnce($WikiMarker, $this->linkPage($WikiWord), $result);
			}
			// Remove escape Character so we don't display it
			preg_match_all("/(\\\)?".self::WikiWordRegex.'/', $result, $EscWikiWords);
			$EscWikiReplacements = str_replace('\\', '', $EscWikiWords[0]);
			$result = str_replace($EscWikiWords[0], $EscWikiReplacements, $result);
		}
	
		if (!$disableBracketLinks) {
			foreach($BracketWords[0] as $BracketWord) {
				$result = StringUtils::strReplaceOnce($BracketMarker, $this->linkPage(substr($BracketWord,1,strlen($BracketWord)-2)), $result);
			}
			// Remove escape Character so we don't display it
			preg_match_all($EscBracketsRegex, $result, $EscBracketWords);
			$EscBracketReplacements = str_replace('\[', '[', $EscBracketWords[0]);
			$result = str_replace($EscBracketWords[0], $EscBracketReplacements, $result);
		}
	
		foreach($HTMLTags[0] as $Tag) {
			$result = StringUtils::strReplaceOnce($HTMLMarker, $Tag, $result);
		}
	
		foreach($Links[0] as $Link) {
			$result = StringUtils::strReplaceOnce($LinkMarker, $Link, $result);
		}
	
		foreach($URLs[0] as $URL) {
			// first and last character must be included too (they were included on the regexp
			// to exclude links inside <a> tags)
			$firstchar = $URL[0];
			$lastchar = $URL[strlen($URL)-1];
			$URL = substr($URL, 1, strlen($URL)-2);
			$result = StringUtils::strReplaceOnce($URLMarker, $firstchar.'<a href="'.$URL.'">'.$URL.'</a>'.$lastchar, $result);
		}
	
		return $result;
	}

	/**
	 * returns a link to the page
	 *
	 * @param string $pagename
	 * @return string
	 */
	private function linkPage($pagename) {
		// show the name underlined with a question mark as link if it doesn't exist. 
		// show the underlined name as a link if it exists
		$parts = explode('|', $pagename, 2);
		$link = trim($parts[0]);
		$linkedText = (isset($parts[1])) ? trim($parts[1]) : $link;

		// Check if it's a GForge page
		/*if ($link{0} == '#') {
			return $this->linkGForgePage($link, $linkedText);
		} else */
		if ($this->pageExists($link)) {
			$linkdisplay = '<u>' . $linkedText . '</u>';
			$before = '';
			$href = GFUrl::getUrl()->currentBaseURL('',array('pagename'=>$link));
		} else {
			$linkdisplay = '?';
			$before = '<u>' . $linkedText . '</u>';
			$href = GFUrl::getUrl()->currentBaseURL('',array('pagename'=>$link,'parent_page'=>$this->pagename));
		}
		return $before.'<a href="'.$href.'">'.$linkdisplay.'</a>';
	}
	
	// This functionality is disabled for user wiki
	/*private function linkGForgePage($link, $linkedText) {
		$link = substr($link, 1);
		$result = '';
		if (strtolower($link{0}) == 'r') {
			// Frs Release
			$link = substr($link, 1);
			$result ='<a href="'.GFUrl::getUrl()->projectURL($this->project, 'FrsReleaseView', 'frs').'&amp;release_id='.$link .'">'.$linkedText.'</a>';
		} else if (strtolower($link{0}) == 'f') {
			// Forum Post
			$link = substr($link, 1);
			$result ='<a href="'.GFUrl::getUrl()->projectURL($this->project, 'ForumBrowse', 'forum').'&amp;_forum_action=ForumMessageBrowse&amp;thread_id='.$link .'">'.$linkedText.'</a>';
			
		} else if (is_numeric($link)) {
			// Tracker Item Id
			$result ='<a href="'.GFUrl::getUrl()->projectURL($this->project, 'TrackerItemEdit', 'tracker').'&amp;tracker_item_id='.$link .'">'.$linkedText.'</a>';
		} else {
			$result = $link.'|'.$linkedText;
		}
		return $result;
	}*/
	
	/**
	 * returns a link to the version. 
	 *
	 * @param unknown_type $pagename
	 * @param unknown_type $version
	 */
	private function linkPageVersion($pagename,$version) {
		$parts = explode('|', $pagename, 2);
		$link = trim($parts[0]);
		// show the version number only
		if ($this->pageExists($link)) {
//		return '<a href="' . GFUrl::getUrl()->currentBaseURL('',array('section'=>$this->section,'ref_id'=>$this->ref_id,'pagename'=>$link,'version'=>$version)) .'">'
//				. $version . '</a>';			
		return '<a href="' . GFUrl::getUrl()->userUrl($this->user, '', 'userwiki', '', array('pagename'=>$link,'version'=>$version)) .'">' . $version . '</a>';
		}
	}

	/**
	 * checks for the existance of a page
	 *
	 * @param string $pagename
	 * @return boolean
	 */
	private function pageExists($pagename) {
		$c = new Criteria();
		$c->add(UserWikiPagePeer::PAGENAME,$pagename);
		$c->add(UserWikiPagePeer::SECTION,$this->section);
		$c->add(UserWikiPagePeer::REF_ID,$this->ref_id);
		if (is_object(UserWikiPagePeer::doSelectOne($c))) {
			return true;
		}
		return false;
		
	}

	/**
	 * the header without a page to reference to
	 *
	 * @param message $text
	 */
	private function generateWikiHeader($text='') {
		$return = '<p><h3>'. $text . '  </h3></p>';
		$return .= '<p />';
		return $return;		
	}
	
	private function generateWikiFooter($text='') {
		$return = '';
		$return .= $text;
		return $return;			
	}
	
	/**
	 * the header for the wiki with a page
	 *
	 * @param string	message (optional)
	 * @return html contents
	 */
	private function generateWikiPageHeader($message='') {
		$components = GFUrl::getUrl()->getURI();
		$arr = explode('/',$components);
		if (in_array('my',$arr)) {
			$subdir = 'my';
		} else {
			$subdir = '';
		}		
		$return = '<p><h3><a href="' . GFUrl::getUrl()->currentBaseURL('',array()).'">
				'.'
				<a href="' . GFUrl::getUrl()->userURL($this->user,'PageRefs',UserwikiPlugin::SECTION,$subdir,array('id'=>$this->page->getPrimaryKey())) .'">'
				. $this->pagename .'</a></h3></p>';
		if ($this->is_admin) {
			// add admin links / functionalities
			$return .= Language::getSessText('UserWiki.AdminMessage') . '<br />';
			if ($this->page->getFlags() == UserWikiPage::FLAGS_LOCKED ) {
				$return .= '<a href="' . GFUrl::getUrl()->currentBaseURL('UnlockPage',array('unlock'=>$this->page->getPrimaryKey())) . '">['. Language::getSessText('UserWiki.Unlock') .']</a>';
			} else {
				$return .= '<a href="' . GFUrl::getUrl()->currentBaseURL('LockPage',array('lock'=>$this->page->getPrimaryKey())) . '">['. Language::getSessText('UserWiki.Lock') .']</a>';
			}
			$return .= ' - - ';
			$return .= '<a href="' . GFUrl::getUrl()->currentBaseURL('RemovePage',array('remove'=>$this->page->getPrimaryKey())) . '">['. Language::getSessText('UserWiki.Remove') .']</a>';
			$return .= ' - - ';
			$return .= '<a href="' . GFUrl::getUrl()->currentBaseURL('',array('_action'=>'updatestatistics')) . '">['. Language::getSessText('UserWiki.UpdateStatistics') .']</a>';
		}
		$return .= '<p />';
		return $return;		
	}
	
	/**
	 * the footer for the wiki with a page
	 * 
	 * @return html contents
	 *
	 */
	private function generateWikiPageFooter() {
		
		$components = GFUrl::getUrl()->getURI();
		$arr = explode('/',$components);
		if (in_array('my',$arr)) {
			$subdir = 'my';
		} else {
			$subdir = '';
		}			
		$return = '';
		
		$buttons = '<p>';
		
		
		if ($this->page->getFlags()==UserWikiPage::FLAGS_UNLOCKED) {
			if($this->page->userIsAdmin()) {
				if ($this->page->getPageName() != 'FrontPage') {
					$deleteURL = GFUrl::getUrl()->userURL($this->user,'RemovePage', UserwikiPlugin::SECTION, $subdir.'/admin', array('remove' => $this->page->getWikiPageId()));
					$buttons .= HTMLWidget::button('button', Language::getSessText('UserWiki.Remove'), '', '', "document.location.href='".$deleteURL."';");
				}
				$lockURL = GFUrl::getUrl()->userURL($this->user,'LockPage', UserwikiPlugin::SECTION, $subdir.'/admin', array('lock' => $this->page->getWikiPageId()));
				$buttons .= HTMLWidget::button('button', Language::getSessText('UserWiki.Lock'), '', '', "document.location.href='".$lockURL."';");
			}
			if($this->page->userCanUpdate()) {
				$editURL = GFUrl::getUrl()->userURL($this->user,'PageEdit',UserwikiPlugin::SECTION,$subdir,array('edit'=>$this->pagename));
				$buttons .= HTMLWidget::button('button', Language::getSessText('UserWiki.EditText'), '', '', "document.location.href='".$editURL."';");
			}
		} else {			
			if($this->page->userIsAdmin()) {
				$lockURL = GFUrl::getUrl()->userURL($this->user,'UnlockPage', UserwikiPlugin::SECTION, $subdir.'admin', array('unlock' => $this->page->getWikiPageId()));
				$buttons .= HTMLWidget::button('button', Language::getSessText('UserWiki.Unlock'), '', '', "document.location.href='".$lockURL."';");
			}
		}
		
		$buttons .= '</p><br /><br />';
		
		if ($this->page_version == $this->page->getLatestVersion()) { // we don't want to edit archived versions			
			if ($this->page->getFlags()==UserWikiPage::FLAGS_UNLOCKED || $this->page->userIsAdmin()) {
				// if the page's not locked
				$return .= $buttons;
			} else {
				// locked page
				if($this->page->userCanUpdate()) {
					$return .= Language::getSessText('UserWiki.LockedPage');
				}
			}
		}

//		$return .= '<div id="mainTabSet" class="tabber">';
//		$return .= '<div class="tabbertab" title="'. Language::getSessText('UserWiki.Versions').'" id="versionstab">';
		$return .= $this->renderVersionsTab();
//		$return .= '</div>';

//		$return .= '</div>';
		return $return;	
	}
	
	/**
	 * generates the html page with the main contents
	 *
	 * @param string $contents
	 * @param string	$message -> optional message in the header
	 * @return html full content
	 */
	private function generatePage($contents,$message='') {
		$return = $this->generateWikiPageHeader($message);
		$return .= $this->Transform($contents); // transform wikilinks, etc
		$return .= $this->generateWikiPageFooter();		
		return $return;
	}

	/**
	 * returns the html for the current page properties. Used by showInfo
	 *
	 * @param int $version : the version id
	 * @return string
	 */
	private function getPageProps($version) {
		// build and return
		$contents = $this->page->getContents($version);
		if ($contents == null) {
			return Language::getSessText('UserWiki.VersionNotExist',array($version));
		} else {
			$return = '<table border="1" bgcolor="white">'."\n";
		}
			
		
		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$sql = 'SELECT page.pagename,page.flags,version.create_date,version.score,version.hits,version.posted_by,version.last_modified_date,version.page_version,version.refs'.
			' FROM wiki_page page, wiki_version version WHERE page.wiki_page_id = \'' . $this->page->getPrimaryKey() . '\' AND version.wiki_page_id = \'' . $this->page->getPrimaryKey() . '\' AND '.
			' version.page_version = \'' . $version . '\'';
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		if ($rs->getRecordCount()==1) {
			$rs->next();
			$values = $rs->getRow();
			if (isset($_REQUEST['showpagesource'])) {
				$refs = $values['refs'];
				$content = $contents = nl2br($contents);
			} else {
				$refs = '';
				$content = '';
			}
			$return .= '<tr><td>'.Language::getSessText('UserWiki.PageName').'</td><td>'.$values['pagename'].'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Version').'</td><td>'.$values['page_version'].'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Flags').'</td><td>'.$values['flags'].'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Author').'</td><td>'.UserPeer::retrieveByPk($values['posted_by'])->getFullName().'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.LastModified').'</td><td>'.$values['last_modified_date'].'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Created').'</td><td>'.$values['create_date'].'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Score').'</td><td>'.$values['score'].'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Hits').'</td><td>'.$values['hits'].'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Refs').'</td><td>'.$refs.'</td></tr>'."\n";
			$return .= '<tr><td>'.Language::getSessText('UserWiki.Content').'</td><td>'.$content.'</td></tr>'."\n";
			$return .= '</table>';
		} else {
			// some error here, should be only one. db corrupt?
			$return = 'ERROR';
		}
		return $return;
	}
	
	/**
	 * returns the information about the page passed as parameter
	 *
	 * @param int $page_name
	 */
	public function showInfo($info) {
		if ((!is_object($this->page)) || ($this->page->getPrimaryKey()!=$info)) { // just a little check to make sure we get the correct page
			if (!$this->retrievePageByName($info)) { // doesn't exist
				return Language::getSessText('UserWiki.PageNotExist',array($info));
			}
		}
		$return = '<p><h3>'. Language::getSessText('UserWiki.PageInfo') . ': ' . $this->pagename . '</h3></p>';
		require_once('plugins/'. UserwikiPlugin::SECTION .'/wwwlib/forms/InfoForm-tpl.php');
		$return .= '<p><b>';
		$return .= Language::getSessText('UserWiki.CurrentVersion');
		$return .= "</b></p>";
		$latest_version_id = $this->page->getLatestVersion()->getPageVersion(); // get the latest version id
		$return .= $this->getPageProps($latest_version_id);
		$return .= '<p><b>';
		$return .= Language::getSessText('UserWiki.PreviousVersion');
		$return .= '</b></p>';
		if ($latest_version_id>1) { // if it's the first one, there's no archived one
			$previous = $this->page->getPreviousVersion();
			if (is_object($previous)) {
				$return .= $this->getPageProps($previous->getPageVersion()); // previous version
			} else {
				$return .= $this->getPageProps(null);
			}
		}
		// TODO -> give an option to view up to 'x' old versions info
		return $return;
	}
	
	/**
	 * returns the diff information about the page passed as parameter against its previous version (if exists)
	 *
	 * @param int $page_id
	 */
	public function showDiff($diff) {
		if ((!is_object($this->page)) || ($this->page->getPrimaryKey()!=$diff)) { // just a little check to make sure we get the correct page
			if (!$this->retrievePageById($diff)) { // doesn't exist
				return Language::getSessText('UserWiki.PageNotExist',$diff);
			}
		}
		$return = '<p><h3>'. Language::getSessText('UserWiki.PageDiff') . ': ' . $this->pagename . '</h3></p>';
		$return .= '<p>';
		$latest = $this->page->getLatestVersion(); // get the latest version id
		$previous = $this->page->getPreviousVersion();
		if ( (is_object($previous)) ) {
			$diff_contents = $this->getDiffContents($previous,$latest);
			$return .= $diff_contents;
		} else {
			$return .= 'No previous version';
		}
		return $return;
	}

	/**
	 * returns the diff information about the page passed as 1st parameter against the one passed as 2nd parameter
	 *
	 * @param int $page_id
	 */
	public function showDiffTo($page, $to) {
		if ((!is_object($this->page)) || ($this->page->getPrimaryKey()!=$page)) { // just a little check to make sure we get the correct page
			if (!$this->retrievePageById($page)) { // doesn't exist
				return Language::getSessText('UserWiki.PageNotExist',$page);
			}
		}
		$return = '<p><h3>'. Language::getSessText('UserWiki.PageDiff').': '.Language::getSessText('UserWiki.DiffLatestTo', array($this->pagename, $to)).'</h3></p>';
		$return .= '<p>';
		
		$latest = $this->page->getLatestVersion();
		$versionTo = $this->page->getVersion($to);
		if ($versionTo instanceof UserWikiVersion) {
			$diff_contents = $this->getDiffContents($latest, $versionTo);
			$return .= $diff_contents;
		} else {
			$return .= 'No such versions';
		}
		return $return;
	}
	
	/**
	 * returns the diff information about the page version passed as 2nd parameter against the one passed as 3rd parameter
	 *
	 * @param int $page_id
	 */
	public function showDiffFromTo($page, $from, $to) {
		if ((!is_object($this->page)) || ($this->page->getPrimaryKey()!=$page)) { // just a little check to make sure we get the correct page
			if (!$this->retrievePageById($page)) { // doesn?t exist
				return Language::getSessText('UserWiki.PageNotExist',$page);
			}
		}
		$return = '<p><h3>'. Language::getSessText('UserWiki.PageDiff').': '.Language::getSessText('UserWiki.DiffFromTo', array($this->pagename, $from, $to)).'</h3></p>';
		$return .= '<p>';
		
		$versionFrom = $this->page->getVersion($from);
		$versionTo = $this->page->getVersion($to);
		if ($versionTo instanceof UserWikiVersion && $versionFrom instanceof UserWikiVersion) {
			$diff_contents = $this->getDiffContents($versionFrom, $versionTo);
			$return .= $diff_contents;
		} else {
			$return .= 'No such versions';
		}
		return $return;
	}
	
	/**
	 * returns a diff between 2 versions of the same page (current page)
	 *
	 * @param version $first
	 * @param version $second
	 */
	private function getDiffContents($first,$second) {
		$return = '';
		if ( (is_object($first) ) && (is_object($second) )) { 
			$first_contents = $this->page->getContents($first->getPageVersion());
			$second_contents = $this->page->getContents($second->getPageVersion());
			// aux files for making the diff
			$fp1 = fopen('/tmp/gforge5_wiki_first_diff','w');
			$fp2 = fopen('/tmp/gforge5_wiki_second_diff','w');
			if ($fp1 && $fp2) {
				fwrite($fp1,$first_contents);
				fwrite($fp2,$second_contents);
				fclose($fp1);
				fclose($fp2);
				exec('diff -u /tmp/gforge5_wiki_first_diff /tmp/gforge5_wiki_second_diff',$output,$ret);
				unlink('/tmp/gforge5_wiki_first_diff');;
				unlink('/tmp/gforge5_wiki_second_diff');;
				foreach ($output as $o) {
					$return .= $o."\n";
				}
			} else {
				$return = 'Couldnt create tmp files for diffing';
			}
		} else {
			$return = 'Invalid version objects';
		}
		$style  = ' style="font-family: Courier New,Courier,monotype; font-size:11px;"';
		$return = str_replace('<br />', "<br />\n", $return);
		$return = str_replace('</p>', "</p>\n", $return);
		$return = str_replace('</div>', "</div>\n", $return);
		$return = '<pre'.$style.'><code>'.htmlentities($return).'</code></pre>';
		return $return;
	}
	
	/**
	 * returns the pages that link to the page passed as parameter
	 *
	 * 
	 * @param int $page_id
	 * @return array	pages -> [scores => array(page,hits,score)]
	 */
	private function getLinkingTo($page_id) {
		$pages = array();
		$c = new Criteria();
		$c->add(UserWikiPageLinkPeer::TO_WIKI_PAGE_ID,$page_id);
		$pages_links = UserWikiPageLinkPeer::doSelect($c);
		foreach ($pages_links as $page_link) {
			$this_page = $page_link->getWikiPageRelatedByFromWikiPageId();
			if (is_object($this_page)) {
				$ver = $this_page->getLatestVersion();
				if (is_object($ver)) {
					// make array and to sort it'll be better for highest score numbers
					$pages[$ver->getScore()] = array($this_page,$ver->getHits(),$ver->getScore());
				}
			}
		}		
		return $pages;
	}

	/**
	 * returns the pages that link from the page passed as parameter
	 *
	 * @param int $page_id
	 * @return array	pages -> [scores => array(page,hits,score)]
	 */
	private function getLinkingFrom($page_id) {
		$pages = array();
		$c = new Criteria();
		$c->add(UserWikiPageLinkPeer::FROM_WIKI_PAGE_ID,$page_id);
		$pages_links = UserWikiPageLinkPeer::doSelect($c);
		foreach ($pages_links as $page_link) {
			$this_page = $page_link->getWikiPageRelatedByToWikiPageId();
			if (is_object($this_page)) {
				$ver = $this_page->getLatestVersion();
				if (is_object($ver)) {
					// make array and to sort it'll be better for highest score numbers
					$pages[$ver->getScore()] = array($this_page,$ver->getHits(),$ver->getScore());
				}
			}
		}
		return $pages;
	}
	
	/**
	 * returns the html for the quick search box
	 *
	 */
	private function getQuickSearchForm() {
		$return = '<form method="POST" action="' . GFUrl::getUrl()->currentBaseURL('Search',array()) . '">';
		$return .= '<input type="text" size="30" name="searchword"/>';
		$return .= '<input type="hidden" name="id" value="'. $this->page->getPrimaryKey() .'"/>';
		$return .= HTMLWidget::button("submit","Search","submit","","");
		$return .= '</form>';
		return $return;
	}
	
	/**
	 * returns the html for the quick search box
	 * 
	 * the search looks for page names with what is passed in the search query
	 * 
	 * @return html code
	 *
	 */
	private function getFullSearchForm() {
		$return = '(not implemented)';
		return $return;
	}
	
	/**
	 * returns the html table with the search results
	 *
	 * @param string $searchwords
	 * @return html code
	 */
	public function getSearchResults($searchwords) {
		$return = '';
		$con = Propel::getConnection(UserWikiPagePeer::DATABASE_NAME);
		$sql = "select pagename,wiki_page_id from wiki_page where ref_id='" . $this->ref_id . "' and section='" . $this->section . "' and pagename ilike '%$searchwords%'"; 
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		$theader = array();
		$theader[] = 'Name';
		$theader[] = 'Versions';
		$return .= $this->generateWikiHeader('Search Results');
		$return .= $this->theme->tableTop($theader);
		if ($rs->getRecordCount()>0) {
			while ($rs->next()) {		
				$trow = array();
				$pagename = $rs->getString('pagename');
				$trow[] = $pagename;
				// get the versions
				$page = UserWikiPagePeer::retrieveByPK($rs->getInt('wiki_page_id'));
				if ($page instanceof UserWikiPage) {
					$versions = $page->getWikiVersions();
					$vers = '';
					foreach ($versions as $version) {
						$vers .= $this->linkPageVersion($pagename,$version->getPageVersion()) . ' | ';
					}
					$vers[strlen($vers)-2] = ' ';
					$trow[] = $vers;
				} else {
					$trow[] = 'Error retrieving versions for this page';
				}
				$return .= $this->theme->tableRow($trow);
			}
		} else {
			$trow[] = 'No matches found';
			$trow[] = 'No matches found';
			$return .= $this->theme->tableRow($trow);
		}
		$return .= $this->theme->tableBottom();
		$return .= $this->generateWikiFooter();
		return $return;
	}

	/**
	 * returns the html table with the most popular pages
	 *
	 * @return html code
	 */
	public function getMostPopular() {
		$return = '';
		$con = Propel::getConnection(UserWikiPagePeer::DATABASE_NAME);
		$sql = "select distinct pagename,wp.wiki_page_id, wp.page_views as hits,wv.page_version from wiki_page wp,wiki_version wv where ref_id='" . $this->ref_id . "' and section='" . $this->section . "' and wp.wiki_page_id = wv.wiki_page_id order by hits desc "; 
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		$theader = array();
		$theader[] = 'Name';
		$theader[] = 'Version';
		$theader[] = 'Hits';
		$return .= $this->theme->tableTop($theader);
		$qty = 1; // display 20
		$shown = array(); // we'll store what pages are already there (only show them once, we don't care if another version one has less hits but it's a high number)
		while (($rs->next()) && ($qty<21)) {		
			$trow = array();
			$pagename = $rs->getString('pagename');
			$trow[] = $pagename;
			if (!in_array($pagename,$shown)) {
				$shown[] = $pagename;
				$page = UserWikiPagePeer::retrieveByPK($rs->getInt('wiki_page_id'));
				if ($page instanceof UserWikiPage) {
					$version = $rs->getInt('page_version');
					$vers = $this->linkPageVersion($pagename,$version);
					$trow[] = $vers;
					$trow[] = $rs->getInt('hits');
				} else {
					$trow[] = 'Error retrieving versions for this page';
					$trow[] = '';
				}
				$return .= $this->theme->tableRow($trow);
			}
			$qty++;
		}
		$return .= $this->theme->tableBottom();
		return $return;
	}	
	
	/**
	 * returns the html code for the references to the page. this is for latest version since the links only apply to it
	 *
	 * @param int $page_id
	 * @param int	page_version
	 */
	public function getPageRefs($page_id) {
		$page = UserWikiPagePeer::retrieveByPK($page_id);
		if ($page instanceof UserWikiPage) {
			$this->page = $page;
			$name = $page->getPagename();
		} else {
			$name = 'ERROR : page not found';
		}
		$return = $this->generateWikiHeader(Language::getSessText('UserWiki.LinkTo',array($name)));
		$links_to = $this->getLinkingTo($page_id);
		$scores = array_keys($links_to);
		rsort($scores);
		reset($scores);
		$i = 0;
		$return .= '<ul>';
		while(($val = each($scores)) && ($i<5)) {
			$item = $links_to[$val[1]][0];
			$return .= '<li>' . $this->linkPage($item->getPageName()) . '</li>';
			$i++;
		}
		$return .= '</ul>' . $this->generateWikiFooter(Language::getSessText('UserWiki.LinkToNumber',array(count($links_to),$name)));
		return $return;
	}
	
	public function setAdmin() {
		$this->is_admin = true;
	}
	
	public function isAdmin() {
		return $this->is_admin;
	}
	
	/**
	 * locks the page
	 *
	 * @param int $pageid
	 */
	public function lockPage($pageid) {
		if ($this->retrievePageById($pageid)) {
			if ($this->page->userIsAdmin()) { // only admins can do this actions
				$this->page->setFlags(UserWikiPage::FLAGS_LOCKED);
				$this->page->save();
				return $this->displayPageById($this->page->getPrimaryKey(),null,'<font color="red">Wiki page locked successfully</font>');
			}
		} else {
			// trying to access by an id which doesn't correspond to the section + ref_id, or else it just doesn't exist
			$ret = $this->generateWikiHeader('<font color=red>'.Language::getSessText('UserWiki.NoAccessThisId').'</font>');
			$ret .= $this->generateWikiFooter();
			return $ret;
		}
	}
	
	/**
	 * unlocks the page
	 *
	 * @param int $pageid
	 */
	public function unlockPage($pageid) {
		if ($this->retrievePageById($pageid)) {
			if ($this->page->userIsAdmin()) { // only admins can do this actions
				$this->page->setFlags(UserWikiPage::FLAGS_UNLOCKED);
				$this->page->save();
				return $this->displayPageById($this->page->getPrimaryKey(),null,'<font color="red">Wiki page unlocked successfully</font>');
			}
		} else {
			// trying to access by an id which doesn't correspond to the section + ref_id, or else it just doesn't exist
			$ret = $this->generateWikiHeader('<font color=red>'.Language::getSessText('UserWiki.NoAccessThisId').'</font>');
			$ret .= $this->generateWikiFooter();
			return $ret;
		}
	}
	
	/**
	 * removes the page and all its versions from the db
	 *
	 * @param int $pageid
	 */
	public function removePage($pageid) {
		if ($this->retrievePageById($pageid)) {
			if ($this->page->userIsAdmin()) { // only admins can do this actions
				// we can't delete the frontpage, check that this is not it
				if ($this->page->getPagename()!='FrontPage') {
					$this->page->delete();
					return $this->displayFrontPage('Page deleted successfully');
				} else {
					return $this->displayFrontPage('Cannot delete the Front Page');
				}
			}
		} else {
			// trying to access by an id which doesn't correspond to the section + ref_id, or else it just doesn't exist
			$ret = $this->generateWikiHeader('<font color=red>'.Language::getSessText('UserWiki.NoAccessThisId').'</font>');
			$ret .= $this->generateWikiFooter();
			return $ret;
		}		
	}
	
	/**
	 * Creates a default set of pages
	 *
	 */
	private function generateDefaults() {
		$dummy = new UserWikiPage();
		$dummy->setRefId($this->ref_id);
		$dummy->setSection('user');
		if ($dummy->userCanCreate()) {
			// check that the user uses the plugin
			if ($this->user->usesPlugin('userwiki')) {			
				if (strlen(Language::getSessText('UserWiki.Defaults.FrontPage'))>1) { // check that at least the frontpage has something, otherwise we assume the cache for the pages is empty
					$this->createNewPage('FrontPage',Language::getSessText('UserWiki.Defaults.FrontPage'));
					$this->createNewPage('Wiki',Language::getSessText('UserWiki.Defaults.Wiki'));
					$this->createNewPage('HowToUseWiki',Language::getSessText('UserWiki.Defaults.HowToUseWiki'));
					$this->createNewPage('AddingPages',Language::getSessText('UserWiki.Defaults.AddingPages'));
					$this->createNewPage('SandBox',Language::getSessText('UserWiki.Defaults.SandBox'));
					$this->createNewPage('RecentVisitors',Language::getSessText('UserWiki.Defaults.RecentVisitors'));
					$this->createNewPage('MostPopular',Language::getSessText('UserWiki.Defaults.MostPopular'));
					$this->updateStatistics();
					return $this->displayFrontPage('<font color="red">Importing first-time defaults</font>');
				} else {
					$return = $this->generateWikiHeader('<font color="red">No defaults generated. Please run create_lang_cache first</font>');
					$return .= $this->generateWikiFooter();
					return $return;			
				}
			} else {
				$return = $this->generateWikiHeader('<font color="red">Please enable this plugin for this user first</font>');
				$return .= $this->generateWikiFooter();
				return $return;					
			}
		} else {
			$return = $this->generateWikiHeader('<font color="red">No defaults generated. An admin must enter the wiki in order to auto-generate default pages</font>');
			$return .= $this->generateWikiFooter();
			return $return;
		}
	}
	
	/**
	 * Recalculates statistic information (incoming links, outgoing links, most visited) for all pages
	 *
	 */
	public function updateStatistics() {
		$c = new Criteria();
		$c->add(UserWikiPagePeer::SECTION,$this->section);
		$c->add(UserWikiPagePeer::REF_ID,$this->ref_id);
		$pages = UserWikiPagePeer::doSelect($c);
		foreach ($pages as $page) {
			$this->updateLinks($page,$page->getLatestContents()); // statistics are always done on latest versions
		}
	}
	
	private function removeEscaped($LinkWords, $simple = true) {
		$result = array();
		if ($simple) {
			$result = array();
			foreach ($LinkWords as $Word) {
				if (strlen($Word) > 1 && $Word{0} != '\\') {
					$result[] = $Word;
				}
			}
		} else {
			$result[0] = array();
			$result[1] = array();
			$result[2] = array();
			for($i=0;$i<count($LinkWords[0]);$i++) {
				$Word = $LinkWords[0][$i];
				if (strlen($Word) > 1 && $Word{0} != '\\') {
					$result[0][] = $Word;
					$result[1][] = $LinkWords[1][$i];
					$result[2][] = $LinkWords[2][$i];
				}
			}
		}
		return $result;
	}

	private function getParents($WikiPage, $Result) {
		$c = new Criteria();
		$c->add(WikiPageLinkPeer::TO_WIKI_PAGE_ID, $WikiPage->getPrimaryKey());
		$c->add(WikiPageLinkPeer::FROM_WIKI_PAGE_ID, $WikiPage->getPrimaryKey(), Criteria::LESS_THAN);
		$c->addDescendingOrderByColumn(WikiPageLinkPeer::FROM_WIKI_PAGE_ID);
		$WikiLink = WikiPageLinkPeer::doSelectOne($c);
		if (is_object($WikiLink)) {
			$WikiLinkWikiPage = $WikiLink->getWikiPageRelatedByFromWikiPageId();
			if ($WikiLinkWikiPage instanceof WikiPage) {
				if (!isset($Result[$WikiLinkWikiPage->getPrimaryKey()])) {
					$Result[$WikiLinkWikiPage->getPrimaryKey()] = $WikiLinkWikiPage->getPagename();
					return $this->getParents($WikiLinkWikiPage, $Result);
				}
			}
		}
		return $Result;
	}
	
	public function getBreadCrumbs() {
		$result = array();
		if (isset($this->page)) {
			$links = $this->getParents($this->page, array());
			foreach($links as $id => $name) {
				$result[] = '<a href="' . GFUrl::getUrl()->currentBaseURL('',array('pagename'=>$name)) .'">'.$name.'</a>';
			}
		}
		
		return $result;
	}
	
	public function getTitle() {
		if (isset($this->page)) {
			return $this->page->getPagename();
		} else {
			return Language::getSessText('Wiki.TabString');
		}
	}
	
	private function renderVersionsTab() {
		$Lang = Session::getSession()->getLanguage();
		$WikiVersions = $this->page->getAllVersions();
		$UsersCache = array();

		$components = GFUrl::getUrl()->getURI();
		$arr = explode('/',$components);
		if (in_array('my',$arr)) {
			$subdir = 'my';
		} else {
			$subdir = '';
		}		
	
		$theader=array();
		$theader[] = $Lang->getText('UserWiki.Version');
		$theader[] = $Lang->getText('UserWiki.Author');
		$theader[] = $Lang->getText('UserWiki.LastModified');
		/*$theader[] = $Lang->getText('UserWiki.Score');*/
	//	$theader[] = $Lang->getText('UserWiki.Hits');
		$theader[] = $Lang->getText('UserWiki.DiffTitle');
		$return = $this->theme->tableTop($theader);
		
		$ThisVersion = (isset($this->page_version)) ? $this->page_version->getPageVersion() : -1;
		$LatestVersion = $this->page->getLatestVersion()->getPageVersion();
		
		$AllVersionNumbers = array();
		for($i=0;$i<count($WikiVersions);$i++) {
			$AllVersionNumbers[$i] = $WikiVersions[$i]->getPageVersion();
		}
		
		$i=0;
		foreach ($WikiVersions as $WikiVersion) {
			if (!isset($UsersCache[$WikiVersion->getPostedBy()])) {
				$UsersCache[$WikiVersion->getPostedBy()] = UserPeer::retrieveByPk($WikiVersion->getPostedBy());
			}
			$trow = array();
			if ($ThisVersion==$WikiVersion->getPageVersion()) {
				$trow[] = '#'.$WikiVersion->getPageVersion();
			} else {
				$trow[] = '<a href="'.GFUrl::getUrl()->currentBaseURL('',array('pagename'=>$this->page->getPagename(),'version'=>$WikiVersion->getPageVersion())) .'">#'.$WikiVersion->getPageVersion().'</a>';
			}
			$trow[] = (is_object($UsersCache[$WikiVersion->getPostedBy()])) ? $UsersCache[$WikiVersion->getPostedBy()]->getFullName() : '';
			$trow[] = $WikiVersion->getLastModifiedDate();
			/*$trow[] = $WikiVersion->getScore();*/
	//		$trow[] = $WikiVersion->getHits();
			
			$prev_version = (isset($AllVersionNumbers[$i+1])) ? $AllVersionNumbers[$i+1] : 0;

			if ($prev_version == 0) {
				$trow[] = '&nbsp;';
			} else {
				$trow[] = '<a href="'.GFUrl::getUrl()->userURL($this->user,'PageDiff',UserwikiPlugin::SECTION,$subdir,array('section'=>$this->section,'ref_id'=>$this->ref_id,'diff'=>$this->page->getPrimaryKey(),'from'=>$prev_version,'to'=>$WikiVersion->getPageVersion())).'">'.$Lang->getText('UserWiki.ShowDiff').'</a>';
			}

			$return .= $this->theme->tableRow($trow);
			$i++;
		}
		$return .= $this->theme->tableBottom();
		return $return;
	}
}

?>
